//---------------------------------------------------------------------------
#ifndef RichEdExH
#define RichEdExH
//---------------------------------------------------------------------------
#include <SysUtils.hpp>
#include <Controls.hpp>
#include <Classes.hpp>
#include <Forms.hpp>
#include <ComCtrls.hpp>
#include <StdCtrls.hpp>

#include <ole2.h>
#include <richole.h>
#include <oledlg.h>

// les diffrences entre les versions du compilateur
#if (__BORLANDC__ == 0x520)
#   define PACKAGE
#   define _BCVERSION_ 1
#else
#   define _BCVERSION_ 3
#endif

// exception gnre par RichEdEx
class ERichEditEx : public Exception
{
public:
   __fastcall ERichEditEx(const AnsiString p0) : Exception(p0) { }
};

// dclaration forward de la classe du composant
class TRichEditEx;


class TRichEditOleCallback : public IRichEditOleCallback
{
protected:
    ULONG lRef;
    TRichEditEx* cControl;      // le composant associ

public:
	// Constructors and Destructor
	TRichEditOleCallback(TRichEditEx* aRichEdit)
        : lRef(0), cControl(aRichEdit) {};
	virtual ~TRichEditOleCallback() {};

public:
	// IUnknown Interface Members
    STDMETHODIMP QueryInterface(THIS_ REFIID riid, LPVOID FAR * lplpObj);
	STDMETHODIMP_(ULONG) AddRef(void);
	STDMETHODIMP_(ULONG) Release(void);

	// IRichEditOleCallback Interface Members
	STDMETHODIMP GetNewStorage(LPSTORAGE* ppStg);
	STDMETHODIMP GetInPlaceContext(LPOLEINPLACEFRAME* ppFrame,
								   LPOLEINPLACEUIWINDOW* ppDoc,
								   LPOLEINPLACEFRAMEINFO pFrameInfo);
	STDMETHODIMP ShowContainerUI(BOOL fShow);
	STDMETHODIMP QueryInsertObject(LPCLSID pclsid, LPSTORAGE pStg, LONG cp);
	STDMETHODIMP DeleteObject(LPOLEOBJECT pOleObj);
	STDMETHODIMP QueryAcceptData(LPDATAOBJECT pDataObj, CLIPFORMAT* pcfFormat,
								 DWORD reco, BOOL fReally, HGLOBAL hMetaPict);
	STDMETHODIMP ContextSensitiveHelp(BOOL fEnterMode);
	STDMETHODIMP GetClipboardData(CHARRANGE* pchrg, DWORD reco,
								  LPDATAOBJECT* ppDataObject);
	STDMETHODIMP GetDragDropEffect(BOOL fDrag, DWORD grfKeyState, LPDWORD pdwEffect);
	STDMETHODIMP GetContextMenu(WORD seltype, LPOLEOBJECT pOleObj, CHARRANGE* pchrg, HMENU* phMenu);
};



//---------------------------------------------------------------------------
class PACKAGE TRichEditEx : public TRichEdit
{
private:
    AnsiString FContainerName;
    AnsiString FObjectMenuPrefix;
    TStringList* FVerbs;
    IRichEditOle* pRichEditOle;
    TRichEditOleCallback* pRichEditCallback;

protected:
    void __fastcall WMCreate(TWMCreate* Message);
    void __fastcall WMDestroy(TWMNoParams* Message);
    void __fastcall CreateParams(TCreateParams &Params);
    HRESULT __fastcall GetNewStorage(LPSTORAGE* ppStg);
    friend STDMETHODIMP TRichEditOleCallback::GetNewStorage(LPSTORAGE* ppStg);
    void __fastcall VerbMenuItemClick(TObject *Sender);

    BEGIN_MESSAGE_MAP
        MESSAGE_HANDLER(WM_CREATE, TWMCreate*, WMCreate)
        MESSAGE_HANDLER(WM_DESTROY, TWMNoParams*, WMDestroy)
    END_MESSAGE_MAP(TComponent)

public:
    __fastcall TRichEditEx(TComponent* Owner);
	__fastcall virtual ~TRichEditEx();

	HRESULT __fastcall SetHostNames(AnsiString ContainerApp, AnsiString ContainerObject);
	void __fastcall SetContainerName(AnsiString cn);
	HRESULT __fastcall InsertObject();
	HRESULT __fastcall InsertObjectFromFile(String fn, bool bLink);
	HRESULT __fastcall CreateNewObject(String ty);
	HRESULT __fastcall PasteSpecial();
    bool __fastcall CanPaste();
    int __fastcall GetSelectionType();
    int __fastcall FillListVerbs();
    HRESULT __stdcall GetObject(IOleObject** ppOleObj);

    AnsiString __fastcall GetUserType(int form);
    AnsiString __fastcall GetObjectName();
    AnsiString __fastcall GetObjectShortName();
    AnsiString __fastcall GetObjectAppName();

    void __fastcall FillPopup(TMenuItem* mi);
    HRESULT __fastcall DoVerb(LONG v);

    __property TStringList* Verbs = {read=FVerbs};

__published:
    __property AnsiString ContainerName = {read=FContainerName, write=SetContainerName};
    __property AnsiString ObjectMenuPrefix = {read=FObjectMenuPrefix, write=FObjectMenuPrefix};
};
//---------------------------------------------------------------------------
#endif

